@@ -4,8 +4,12 @@ from datetime import datetime, timedelta, date
 from decimal import Decimal
 from os.path import join
 from pprint import pprint
+from urllib.parse import urlparse
 from django.utils import timezone
 from typing import List
+
+from django.utils.timezone import now
+
 from jutil.modelfields import SafeCharField, SafeTextField
 from jutil.middleware import logger as jutil_middleware_logger, ActivateUserProfileTimezoneMiddleware
 import pytz
@@ -27,10 +31,10 @@ from jutil.command import get_date_range_by_name, add_date_range_arguments, pars
 from jutil.dict import dict_to_html, choices_label
 from jutil.email import make_email_recipient_list
 from jutil.middleware import EnsureOriginMiddleware, LogExceptionMiddleware, EnsureLanguageCookieMiddleware
-from jutil.model import is_model_field_changed, clone_model, get_model_field_label_and_value, get_object_or_none
+from jutil.model import is_model_field_changed, clone_model, get_model_field_label_and_value, get_object_or_none, \
+    wait_object_or_none
 from jutil.request import get_ip_info
 from jutil.responses import FileSystemFileResponse, CsvResponse
-from jutil.sftp import parse_sftp_connection
 from jutil.testing import DefaultTestSetupMixin
 from jutil.urls import url_equals, url_mod, url_host
 from jutil.xml import xml_to_dict, dict_to_element, _xml_filter_tag_name
@@ -47,7 +51,7 @@ from jutil.validators import fi_payment_reference_number, se_ssn_validator, se_s
     dk_iban_bank_info, dk_clearing_code_bank_name, country_code_sanitizer, phone_sanitizer, email_sanitizer, \
     fi_company_org_id_generator, phone_validator, passport_filter, passport_validator, passport_sanitizer, \
     country_code_validator, validate_country_iban, iban_bic, validate_country_company_org_id, fi_ssn_generator, \
-    fi_ssn_validator, bic_validator, iban_generator
+    fi_ssn_validator, bic_validator, iban_generator, bic_sanitizer, filter_country_company_org_id
 from xml.etree.ElementTree import Element
 from xml.etree import ElementTree as ET
 from django.contrib import admin
@@ -178,17 +182,23 @@ class Tests(TestCase, DefaultTestSetupMixin):
 
     def test_passport(self):
         self.assertEqual(passport_filter('?ADsd-12312dsds'), 'ADSD-12312DSDS')
+        passport_validator('21412312312')
         with self.assertRaisesMessage(ValidationError, _('Invalid passport number')):
             passport_validator('214')
         self.assertEqual(passport_sanitizer('214'), '')
+        self.assertEqual(passport_sanitizer('21412312312'), '21412312312')
 
     def test_country_code(self):
+        for cc in ['FI', 'DK', 'ES', 'SE', 'VN']:
+            country_code_validator(cc)
         with self.assertRaisesMessage(ValidationError, _('Invalid country code')):
             country_code_validator('Finland')
 
     def test_bic(self):
         bic = iban_bic('FI21 1234 5600 0007 85')
         self.assertEqual(bic, 'NDEAFIHH')
+        self.assertEqual(bic_sanitizer('NDEAFIHH'), 'NDEAFIHH')
+        self.assertEqual(bic_sanitizer('NDEAFIH'), '')
 
     def test_org_id(self):
         try:
@@ -204,21 +214,38 @@ class Tests(TestCase, DefaultTestSetupMixin):
 
     def test_fi_ssn_generator(self):
         self.assertEqual(len(fi_ssn_generator()), 6 + 1 + 4)
-        for n in range(10):
-            ssn = fi_ssn_generator()
-            try:
-                fi_ssn_age(ssn)
+        for min_year, max_year in [(1800, 1900), (1900, 2000), (2000, 2050)]:
+            for n in range(10):
+                ssn = fi_ssn_generator(min_year, max_year)
+                try:
+                    fi_ssn_age(ssn)
+                    fi_ssn_validator(ssn)
+                except Exception:
+                    self.fail('{} is valid SSN'.format(ssn))
+        fi_ssn_validator('110305+283X')
+        for ssn in ['9999-123F', '271138-670X', '090228+256X']:
+            with self.assertRaisesMessage(ValidationError, _('Invalid personal identification number')):
                 fi_ssn_validator(ssn)
-            except Exception:
-                self.fail('{} is valid SSN'.format(ssn))
+        with self.assertRaises(ValidationError):
+            fi_ssn_generator(1700, 1799)
+        with self.assertRaises(ValidationError):
+            fi_ssn_generator(2100, 2200)
 
     def test_iban(self):
+        with self.assertRaisesMessage(ValidationError, _('Invalid IBAN account number')):
+            iban_validator('')
+        with self.assertRaisesMessage(ValidationError, _('Invalid country code')):
+            iban_validator('XX')
         with self.assertRaisesMessage(ValidationError, _('Invalid IBAN account number')):
             iban_validator('FI2112345600000786')
         bic_validator('HELSFIHH')
         bic_validator('HELSFIHHXXX')
         with self.assertRaisesMessage(ValidationError, _('Invalid bank BIC/SWIFT code')):
             bic_validator('HELSFIH')
+        with self.assertRaisesMessage(ValidationError, _('Invalid bank BIC/SWIFT code')):
+            bic_validator('')
+        with self.assertRaisesMessage(ValidationError, _('Invalid bank BIC/SWIFT code')):
+            bic_validator('XX123123123112')
         iban_validator('FI2112345600000785')
         iban_validator('SE4550000000058398257466')
         fi_iban_validator('FI2112345600000785')
@@ -231,6 +258,7 @@ class Tests(TestCase, DefaultTestSetupMixin):
             se_iban_validator('SE4550000000058398257465')
         iban = 'FI8847304720017517'
         self.assertEqual(iban_filter_readable(iban), 'FI88 4730 4720 0175 17')
+        self.assertEqual(iban_filter_readable(''), '')
 
     def test_urls(self):
         url = 'http://yle.fi/uutiset/3-8045550?a=123&b=456'
@@ -456,16 +484,25 @@ class Tests(TestCase, DefaultTestSetupMixin):
         ]
         invalids = [
             '2084069-1',
+            'SE2084069-1',
         ]
+        co_filtered = [
+            ('FI', 'FI01098230', '0109823-0'),
+            ('FI', 'FI-01098230', '0109823-0'),
+            ('FI', '0109823-0', '0109823-0'),
+            ('FI', '2084069-9', '2084069-9'),
+            ('SE', '01098230', '01098230'),
+            ('SE', '20840699', '20840699'),
+        ]
+        for cc, org, val in co_filtered:
+            self.assertEqual(filter_country_company_org_id(cc, org), val)
+            validate_country_company_org_id(cc, org)
         for valid in valids:
-            # print('test_org_id_fi:', valid, 'should be valid...', end=' ')
             fi_company_org_id_validator(valid)
-            # print('ok')
         for invalid in invalids:
             try:
-                # print('test_org_id_fi:', invalid, 'should be invalid', end=' ')
                 fi_company_org_id_validator(invalid)
-                self.assertTrue(False)
+                self.fail('{} passed as valid FI-org'.format(invalid))
             except ValidationError:
                 # print('ok')
                 pass
@@ -503,6 +540,13 @@ class Tests(TestCase, DefaultTestSetupMixin):
         for ref_no in valid_iso_refs:
             iso_payment_reference_validator(ref_no)
 
+        invalid_iso_refs = [
+            'RF92 1229}',
+        ]
+        for ref_no in invalid_iso_refs:
+            with self.assertRaisesMessage(ValidationError, 'Invalid payment reference'):
+                iso_payment_reference_validator(ref_no)
+
     def test_fi_ssn_age(self):
         samples = [
             (date(2018, 12, 20), '231298-965X', 19),
@@ -518,6 +562,12 @@ class Tests(TestCase, DefaultTestSetupMixin):
     def test_se_banks(self):
         self.assertEqual(se_clearing_code_bank_info('6789'), ('Handelsbanken', 9))
         se_iban_validator('SE45 5000 0000 0583 9825 7466')
+        with self.assertRaisesMessage(ValidationError, _('Invalid IBAN account number')):
+            se_iban_validator('')
+        with self.assertRaisesMessage(ValidationError, _('Invalid IBAN account number')):
+            se_iban_validator('XX45 5000 0000 0583 9825 7466')
+        with self.assertRaisesMessage(ValidationError, _('Invalid IBAN account number')):
+            se_iban_validator('SE45 5000 0000 0583 9825')
         self.assertEqual(se_clearing_code_bank_info('9500'), ('Nordea AB', 10))
         an = '957033025420'
         bank_name, acc_digits = se_clearing_code_bank_info(an)
@@ -585,19 +635,15 @@ class Tests(TestCase, DefaultTestSetupMixin):
 
     def test_parse_sftp(self):
         test_cases = [
-            ('jani@kajala.com', ['jani', '', 'kajala.com', '']),
-            ('jani.kajala:1231!@kajala.com', ['jani.kajala', '1231!', 'kajala.com', '']),
-            ('jani:1231!@kajala.com:/my/dir', ['jani', '1231!', 'kajala.com', '/my/dir']),
-            ('jani.kajala:1231!@kajala.com:my/dir', ['jani.kajala', '1231!', 'kajala.com', 'my/dir']),
-            ('user=jani;host=kajala.com', ['jani', '', 'kajala.com', '']),
-            ('user=jani.kajala;pass=1231!;host=kajala.com', ['jani.kajala', '1231!', 'kajala.com', '']),
-            ('user=jani;pass=1231!;host=kajala.com;path=/my/dir', ['jani', '1231!', 'kajala.com', '/my/dir']),
-            ('user=jani.kajala;pass=1231!;host=kajala.com;path=my/dir',
-             ['jani.kajala', '1231!', 'kajala.com', 'my/dir']),
+            ('sftp://jani@kajala.com', ['jani', None, 'kajala.com', '']),
+            ('sftp://jani.kajala:1231!@kajala.com', ['jani.kajala', '1231!', 'kajala.com', '']),
+            ('sftp://jani:1231!@kajala.com/my/dir', ['jani', '1231!', 'kajala.com', '/my/dir']),
+            ('sftp://jani.kajala:1231!@kajala.com/my/dir', ['jani.kajala', '1231!', 'kajala.com', '/my/dir']),
         ]
         for connection, ref_res in test_cases:
-            res = parse_sftp_connection(connection)
-            self.assertListEqual(list(res), ref_res,
+            res = urlparse(connection)
+            res_list = [res.username, res.password, res.hostname, res.path]
+            self.assertListEqual(res_list, ref_res,
                                  'SFTP connection string "{}" parsed incorrectly'.format(connection))
 
     def test_admin(self):
@@ -800,15 +846,18 @@ class Tests(TestCase, DefaultTestSetupMixin):
         ]
         for iban in test_ibans:
             iban_validator(iban)
-        for n in range(100):
-            acc = iban_generator()
-            # print(acc)
-            try:
-                iban_validator(acc)
-            except Exception as e:
-                print('iban_generator() returned', acc, 'but iban_validator() raised exception', e)
-                self.fail(
-                    'iban_validator(iban_generator()) should not raise Exception, account number was {}'.format(acc))
+        for cc in ['', 'FI', 'SE']:
+            for n in range(100):
+                acc = iban_generator(cc)
+                try:
+                    iban_validator(acc)
+                except Exception as e:
+                    print('iban_generator() returned', acc, 'but iban_validator() raised exception', e)
+                    self.fail('iban_validator(iban_generator()) should not raise Exception, account number was {}'.format(acc))
+        with self.assertRaisesMessage(ValidationError, _('Invalid country code')):
+            iban_generator('XX')
+        with self.assertRaisesMessage(ValidationError, _('IBAN checksum generation does not support >26 character IBANs')):
+            iban_generator('AL')
 
     def test_make_email_recipient(self):
         email_tests = [
@@ -988,6 +1037,20 @@ class Tests(TestCase, DefaultTestSetupMixin):
         self.assertEqual(content_ref, res.content)
         print(res.content.decode())
 
+    def test_wait_object_or_none(self):
+        admin_log([self.user], 'Hello, world')
+        e = LogEntry.objects.all().filter(object_id=self.user.id).last()
+        assert isinstance(e, LogEntry)
+        e_id = e.id
+        obj = wait_object_or_none(LogEntry, id=e_id)
+        self.assertIsNotNone(obj)
+        self.assertEqual(obj.id, e_id)
+        t0 = now()
+        obj = wait_object_or_none(LogEntry, timeout=1.0, sleep_interval=0.1, id=e_id+1)
+        t1 = now()
+        self.assertIsNone(obj)
+        self.assertGreater(t1-t0, timedelta(seconds=0.99))
+
 
 dummy_admin_func_a.short_description = 'A'  # type: ignore
 dummy_admin_func_b.short_description = 'B'  # type: ignore
